classdef PM1000

   properties
   
      model = "PM1000"
      version = "0.9.0"
      
   end

   methods(Static)
   
      %%%%%%%%%%%%%%%%%%%%%%%
      % Basic communication %
      %%%%%%%%%%%%%%%%%%%%%%%
      
      function init(ARG)
         % PM1000.init'192.168.1.100');    opens LAN device
         % PM1000.init(2);                  opens last USB device, with automatic re-connect, no user input
         % PM1000.init(1);                  opens last USB device
         % PM1000.init(0);                  shows list of USB devices and lets user chose one
         % PM1000.init;                     same as pm=PM1000(1);
         
         if (nargin<1)
            ARG=1; % connect to last device
         end;
      
         global PMHandle
         if length(ARG)>1&&sum(class(ARG)=='char')==4  % LAN DEVICE
            PMHandle=NOVTCP(ARG);
         else % USB DEVICE
            [PMHandle, ok] = NOVUSB('LastDevPM.mat', ARG);
            if ARG==2, % automatic re-connect in scripts
               pause(5);
               tries=0;
               while (ok==0)&&(tries<30),
                  pause(1);
                  tries = tries + 1
                  [PMHandle, ok] = NOVUSB('LastDevPM.mat', ARG);
               end;
            end;
         end;
         
      end;
      
      function close
         global PMHandle
         PMHandle.close; 
      end;
      
      function [res ok]  = read(addr)
      
         global PMHandle
         
         [res ok] = PMHandle.read(addr);
         
         if ok==0, % su success in PM1000.reading. Try to connect again
            PM1000.init(2);
            [res ok] = PMHandle.read(addr);
         end;
         
      end;
      
      function [dout1 ok] = readburst(rdaddr, addrstart, addrstop, wraddr)
	  
         global PMHandle
         [dout1 ok] = PMHandle.readburst(rdaddr, addrstart, addrstop, wraddr);
         if ok==0, % su success in reading. Try to connect again
            EPS1000.init(2);
            [dout1 ok] = PMHandle.readburst(rdaddr, addrstart, addrstop, wraddr);
         end;
		end;

      function ok = write(addr, data)
         global PMHandle
         ok = PMHandle.write(addr, data);
         if ok==0, % su success in writing. Try to connect again
            PM1000.init(2);
            ok = PMHandle.write(addr, data);
         end;
      end;
      
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%
      % General instrument data %
      %%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      function r = getfirmware % as string
         r = sprintf('%4X', PM1000.read(512+128));
      end

      function r = getserialnumber
         r = PM1000.read(512+133);
      end
      
      function r = getmoduletype % as string
         str=[];
         for ii=0:15,
            dummy = PM1000.read(512+144+ii);
            str=[ str char(bitshift(dummy, -8)) ];
            str=[ str mod(dummy, 2^8) ];
         end;
         r = str;
      end
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%
      % Calibration functions %
      %%%%%%%%%%%%%%%%%%%%%%%%%
      
      function setATE(ATE)
         PM1000.write(512+1, ATE);
      end;
      
      function r = getATE
         r = PM1000.read(512+1);
      end;
      
      function setnormmode(norm_mode)
         % 0: Non-normalized
         % 1: Standard normalization
         % 2: Exact normalization3
         norm_mode=min(2, max(0, norm_mode));
         PM1000.write(512+46, norm_mode);
      end;
      
      function r = getnormmode
         r = PM1000.read(512+46);
      end;
      
      function setnormlevel(level_uW) % in microwatts
         %for example: PM1000.write(512+38, 2^11); % Normalize S1-S3 to 2^11=2048 µw
         PM1000.write(512+38, round(level_uW));
      end;
      
      function r = getoptfrequency % in THz
         r = PM1000.read(512+69)/100;
      end;
      
      function setoptfrequency(frequency) % in THz
         r = PM1000.write(512+69, round(frequency*100));
      end;
      
      function r = getmincalfreq % in THz
         r = PM1000.read(512+70)/100;
      end;

      function r = getmaxcalfreq % in THz
         r = PM1000.read(512+71)/100;
      end;
      
      function r = getminfreq % in THz
         r = PM1000.read(512+67)/100;
      end;

      function r = getmaxfreq % in THz
         r = PM1000.read(512+68)/100;
      end;
      
      function setswitchstate(state)
         PM1000.write(512+430, state);
      end;
      
      function setpowerleftshift(bits)
         PM1000.write(512+74, bits);
      end;
      
      function r = getswitchstate
         r = PM1000.read(512+430);
      end;
      
      function r = gettemperature
         r = PM1000.read(512+421)/16;
      end;
      
      function settemperature(tmp)
      
         % setpoint of the heating element
         PM1000.write(512+431, round(tmp*16));
         % enable the heating element
         PM1000.write(512+432, 59362);
         
         % setpoint (max. 63°C) and hysteresis for the cooling fan
         hyst = 0;
         %temp = 63;
         temp = tmp+1;
         PM1000.write(512+433, round(hyst*16)*2^10 + round(temp*16));
      end;
      
      
      
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % Data aquisition functions %
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      
      function r = getphotocurrents
         r=zeros(4,1);
         r(1)=PM1000.read(512+2);
         r(2)=PM1000.read(512+6); % latched signals
         r(3)=PM1000.read(512+7); % latched signals
         r(4)=PM1000.read(512+8); % latched signals
         bitshift=PM1000.read(512+9);
         r=r/2^bitshift;
      end;
      

      
      function s = getStokes(norm)
         s=zeros(4,1);
         switch norm
            case 0 % Non-normalized
               s(1)=PM1000.read(512+38)/2^15;
               s(2)=(PM1000.read(512+42)-2^15)/2^15;
               s(3)=(PM1000.read(512+43)-2^15)/2^15;
               s(4)=(PM1000.read(512+44)-2^15)/2^15;
            case 1 % Standard
               s(1)=PM1000.read(512+24)/2^15; % DOP
               s(2)=(PM1000.read(512+28)-2^15)/2^15;
               s(3)=(PM1000.read(512+29)-2^15)/2^15;
               s(4)=(PM1000.read(512+30)-2^15)/2^15;
            case 2 % Exact
               s(1)=PM1000.read(512+31)/2^15; % DOP
               s(2)=(PM1000.read(512+35)-2^15)/2^15;
               s(3)=(PM1000.read(512+36)-2^15)/2^15;
               s(4)=(PM1000.read(512+37)-2^15)/2^15;
            case 3 % In µW
               s(1)=PM1000.read(512+10) + PM1000.read(512+11)/2^16;
               g = PM1000.read(512+18)-2^15;
               n = bitxor(PM1000.read(512+19), 2^15)/2^16;
               if (g<0), s(2)=g-n; else s(2)=g+n; end;
               g = PM1000.read(512+20)-2^15;
               n = bitxor(PM1000.read(512+21), 2^15)/2^16;
               if (g<0), s(3)=g-n; else s(3)=g+n; end;
               g = PM1000.read(512+22)-2^15;
               n = bitxor(PM1000.read(512+23), 2^15)/2^16;
               if (g<0), s(4)=g-n; else s(4)=g+n; end;
         end
      end;
      
      function r = getDOP
         r = PM1000.read(512+24)/2^15;
      end;
      
      function r = getPower
         r = PM1000.read(512+10) + PM1000.read(512+11)/2^16; % Power in µW
      end;
      
      function ev = getEigen(sel)
         % sel:
         % 1 = eigenvector1, corresp. to strongest eigenvalue
         % 2 = eigenvector2, corresp. to weakest eigenvalue
         % 3 = eigenvector3, cross product of eigenvector1 and eigenvector2
         % 4 = mean Stokes vector
         if (sel>=1)&&(sel<=4)
            ev = zeros(3,1);
            for ii=1:3,
               val = PM1000.read(512+320+ii-1+(sel-1)*3);
               % bit vector to signed
               if val>=2^15, val=val-2^16; end;
               ev(ii) = val;
            end;
            
            % normalization
            ev = ev / norm(ev);
         else
            ev = 0;
         end;
      end;
         
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % SDRAM recording & data transfer %
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      function pclear % clears poincare sphere on hdmi output
         PM1000.write(512+189, 0);
      end;
      
      function setME(ME) % sets Memory Exponent. 
         PM1000.write(512+73, ME);
      end;
      
      function r = getME
         r = PM1000.read(512+73);
      end;
      

      
      function [res, ok] = getsdram(startaddr, numaddr)
      
         global PMHandle
         [res, ok] = PMHandle.readhspm(startaddr, numaddr);
         
         while (ok==0),
            PM1000.init(2);
            [res ok]  = PM1000.read(512);
            if (ok==1),
               [res, ok] = PMHandle.readhspm(startaddr, numaddr); 
            end;
         end;
      end;
      
      function [res, ok] = getsdram_norm(startaddr, numaddr, normalization)
         [res, ok] =  PM1000.getsdram(startaddr, numaddr);
         if normalization==0,
            for ii=1:size(res, 2),
               res(2,ii) = res(2,ii)*2 - 2^16;
               res(3,ii) = res(3,ii)*2 - 2^16;
               res(4,ii) = res(4,ii)*2 - 2^16;
            end;
         elseif normalization==1,
            for ii=1:size(res, 2),
               res(1,ii) = res(1,ii) / 2^15;
               res(2,ii) = (res(2,ii)-2^15) / 2^15;
               res(3,ii) = (res(3,ii)-2^15) / 2^15;
               res(4,ii) = (res(4,ii)-2^15) / 2^15;
            end;
         end;
      end;
      
      
      % Reads one block of the Poincaré sphere frame buffer
      function [res ok] = readfr(BlockNr)
         global PMHandle
         [res, ok] = PMHandle.readfr(BlockNr);
      end;



   end;

end